% Decision level fusion from all individual one-against-all classifiers
% using feature selection and upsampling
% (boredom, Happiness, Anger, Sadness, Neutral, and Fear) 
%
% See the Bridge project website at Wireless Communication and Networking
% Group (WCNG), University of Rochester for more information: 
% http://www.ece.rochester.edu/projects/wcng/project_bridge.html
%
% Written by Jianbo Yuan and Yun Zhou, University of Rochester.
% Copyright (c) 2014 University of Rochester.
% Version: June 2014.
%
% Permission to use, copy, modify, and distribute this software without 
% fee is hereby granted FOR RESEARCH PURPOSES only, provided that this
% copyright notice appears in all copies and in all supporting 
% documentation, and that the software is not redistributed for any fee.
%
% For any other uses of this software, in original or modified form, 
% including but not limited to consulting, production or distribution
% in whole or in part, specific prior permission must be obtained from WCNG.
% Algorithms implemented by this software may be claimed by patents owned 
% by WCNG, University of Rochester.
%
% The WCNG makes no representations about the suitability of this 
% software for any purpose. It is provided "as is" without express
% or implied warranty.
%

clear all;
close all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Parameter settings
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Absolute confidence threshold setting. If the highest confidence scores are 
% smaller than 'thresh', the sample is left to be unclassified. A larger 
% 'thresh' value can increase the emotion classification accuracy at the 
% expense of rejecting more samples as unclassified.

featureSet = [80]; % number of features selected

for m = 1:length(featureSet)
    
    thresh = -2:0.05:2.0; % absolute confidence threshold
    cnt_thresh = length(thresh);
    no_of_class = 7; % number of emotion classes
    num_folds = 7; % number of rounds of cross-validations
    num_trials = 5; % number of trials for upsampling
    % % using mutual information to generate different feature sets
    featureMat = ['ldc_clean_top' num2str(featureSet(m))]; % .mat generated by mutualInfo.m
    load(featureMat);
    feature = eval(['top' num2str(featureSet(m))]); % select feature column indices in Excel sheets

    % % random selected feature
    % feature = sort(randperm(121,featureSet(m))) + 3;
    
    dataset = 'uneven_male'; % Select dataset (even or uneven) 
    dirData = '../Datasets/Dataset_Berlin_Clean_norm'; % folder for partitioned data
    
    trainfilepath = [dirData '/train_' dataset]; % path where the training data is located
    testfilepath = [dirData '/test_male']; % path where the testing data is located
    xlsname = '../Excels/raw_OAA_male.xls'; % an output Excel file containing the result
    sheet = ['Sheet' num2str(m)];
    offset = 1; % which line to write in the output excel file

    % store the result for changing threshold for each CV fold
    % 1 - absolute threshold, 2 - rejection rate, 3 - correct classification rate
    Thresh_accu{num_folds,1} = zeros(3,cnt_thresh);

    % DL recall for each individual emotion - anger, sad, boredom, neutral, happy, fear
    recall_emotion = zeros(num_folds, no_of_class);

    % num of test, rejection, confusion matrix
    num_test = zeros(num_folds,1);
    num_rej = zeros(num_folds,1);
    num_confu = zeros(num_folds,1);
    for time = 1:num_trials % number of trials for upsampling
        disp([num2str(time) ' upsampling trial']);
        % 7 fold cross-validation
        for cnt = 1:num_folds
            disp(['Running cross validation fold ' num2str(cnt) '...']);
            disp('----------------------------------');
            [Thresh_accu{cnt,1}, recall_emotion(cnt,:), num_emotion] = OnefoldCV(trainfilepath, ...
                testfilepath, feature, thresh, dataset, cnt);
        end
        % Calculate average recall for each emotion over k folds - happy, sad, fear, anger,
        % boredom, neutral
        recall_emotion(num_folds+1,:) = mean(recall_emotion(1:num_folds,:));
        % Calculate average rejection rate, correct classification rate over k folds
        if cnt_thresh == 1
            avg_rej_rate = zeros(1,cnt_thresh);
            avg_cor_rate = zeros(1,cnt_thresh);
            for myi = 1:cnt_thresh
                for cnt = 1:num_folds
                    avg_rej_rate(myi) = avg_rej_rate(myi)+Thresh_accu{cnt,1}(2,myi);
                    avg_cor_rate(myi) = avg_cor_rate(myi)+Thresh_accu{cnt,1}(3,myi);
                end
            end
            avg_rej_rate = avg_rej_rate./num_folds;
            avg_cor_rate = avg_cor_rate./num_folds;
        else
            % construct a (2*num_fold) * cnt_thresh matrix with (2*j-1)th row being the rejection rate,
            % (2*j)th row being the correct classification rate (j = 1:7)
            rej_cor = zeros(2*num_folds, cnt_thresh);
            for cnt = 1:num_folds
                rej_cor(2*cnt-1,:) = Thresh_accu{cnt,1}(2,:);
                rej_cor(2*cnt,:) = Thresh_accu{cnt,1}(3,:);
            end
            % apply linear interpolation to obtain the average rejection rate and correct
            % classification rate
            [avg_rej_rate, avg_cor_rate] = interp(rej_cor,num_folds);
         end
 
%         % plot decision-level rejection rate vs. correct classification rate
%         fig1 = figure;
%         hold on;
%         plot(avg_rej_rate,avg_cor_rate,'-sr','LineWidth',2,'MarkerSize',10);
%         h1 = xlabel('Rejection rate (%)'); 
%         h2 = ylabel('Decision-level correct classification rate (%)');
%         set(h1,'fontsize',12);
%         set(h2,'fontsize',12);
%         set(gca,'fontsize',12);
%         xlim([0 80]);
%         ylim([45 100]);
%         grid on;
%         pause(0.2);
        
        % % Write to an excel file to store the result:
        % 1st row. absolute confidence threshold, 2nd row. average rejection rate, 
        % 3rd row. correct classification rate
        warning off MATLAB:xlswrite:AddSheet
        xlswrite(xlsname, thresh, sheet, sprintf('A%d',offset));
        offset = offset+1;
        xlswrite(xlsname, avg_rej_rate, sheet, sprintf('A%d',offset));
        offset = offset+1;
        xlswrite(xlsname, avg_cor_rate, sheet, sprintf('A%d',offset));
        offset = offset+1;

        % Display average rejection rate and correct classification rate for each
        % absolute confidence threshold
        disp('Absolute threshold:');
        disp(thresh);
        disp('Avg rejection rate:');
        disp(avg_rej_rate);
        disp('Correct classification rate:');
        disp(avg_cor_rate);
        
        % Display decision-level recall for each individual emotion at thresh = 0
        if cnt_thresh == 1
            disp('Recall for each emotion:');
            fprintf('\tdisgust\t\thappy\t\tanger\tsad\t\tneutral\tboredom\t\tfear\n');
            disp(recall_emotion(num_folds+1,:));
        end           
        disp('-------------------------------------------------------------');
    end
    close all;
    disp([num2str(featureSet(m)) ' feature DONE']);
    fprintf('\n');
end